<?php defined('SYSPATH') or die('No direct access allowed.');

/**
 * This class holds our LDAP Schema
 *
 * @package    LDAP
 * @category   Schema
 * @author     Deon George
 * @copyright  (c) 2013 phpLDAPadmin Development Team
 * @license    http://dev.phpldapadmin.org/license.html
 */
abstract class Kohana_Schema {
	// The schema item's name.
	protected $name = null;
	// The OID of this schema item.
	private $oid = null;
	// The description of this schema item.
	protected $description = null;
	// Boolean value indicating whether this objectClass is obsolete
	private $is_obsolete = false;
	// Seconds to cache our schema details for
	protected $_cache_time = 86400;

	public function getDescription() {
		return $this->description;
	}

	/**
	 * Gets whether this objectClass is flagged as obsolete by the LDAP server.
	 */
	public function getIsObsolete() {
		return $this->is_obsolete;
	}

	/**
	 * Return the objects name.
	 *
	 * param boolean $lower Return the name in lower case (default)
	 * @return string The name
	 */
	public function getName($lower=true) {
		return $lower ? strtolower($this->name) : $this->name;
	}

	public function getOID() {
		return $this->oid;
	}

	public function setDescription($desc) {
		$this->description = $desc;
	}

	/**
	 * Sets this attriute's name.
	 *
	 * @param string $name The new name to give this attribute.
	 */
	public function setName($name) {
		$this->name = $name;
	}

	public function setOID($oid) {
		$this->oid = $oid;
	}
}
?>
